@@ -87,16 +87,24 @@ pub trait IntoAngle<To>
 }
 
 /// Base functionality for all angle types.
-pub trait Angle: Clone + FromAngle<Self> {
+pub trait Angle: Clone + FromAngle<Self> + PartialEq + PartialOrd + num::Zero {
     /// Internal type storing the angle value.
     type Scalar: Float;
 
+    /// Construct a new angle.
+    ///
+    /// Equivalent to constructing the tuple struct directly, eg. `Deg(value)`,
+    /// but usable in a generic context.
+    fn new(value: Self::Scalar) -> Self;
+
     /// The length of a full rotation.
     fn period() -> Self::Scalar;
     /// Return the scalar (unitless) value.
     ///
     /// Equivalent to `self.0` or to doing `let Deg(val) = self`
     fn scalar(&self) -> Self::Scalar;
+    /// Set the internal scalar value of the angle.
+    fn set_scalar(&mut self, value: Self::Scalar);
     /// Normalize the angle, wrapping it back into the standard domain.
     ///
     /// After normalization, an angle will be in the range `[0, self.period())`.
@@ -104,7 +112,7 @@ pub trait Angle: Clone + FromAngle<Self> {
     /// For performance reasons, normalization does not happen automatically
     /// during most operations. Thus, when passing an angle to a method that
     /// expects it to be within the standard domain, first normalize the angle.
-    fn normalize(&self) -> Self;
+    fn normalize(self) -> Self;
     /// Whether the angle is in the standard domain.
     fn is_normalized(&self) -> bool;
 
@@ -172,18 +180,14 @@ pub trait Interpolate: Angle {
 
 macro_rules! impl_angle {
     ($Struct: ident, $period: expr) => {
-        impl<T: Float> $Struct<T> {
-            /// Construct a new angle. 
-            /// 
-            /// Equivalent to constructing the tuple struct directly, eg. `Deg(value)`.
-            pub fn new(value: T) -> $Struct<T> {
-                $Struct(value)
-            }
-        }
-
         impl<T: Float> Angle for $Struct<T>
         {
             type Scalar = T;
+
+            fn new(value: T) -> $Struct<T> {
+                $Struct(value)
+            }
+
             fn period() -> T {
                 cast($period).unwrap()
             }
@@ -191,11 +195,14 @@ macro_rules! impl_angle {
             fn scalar(&self) -> T {
                 self.0
             }
+            fn set_scalar(&mut self, value: T) {
+                self.0 = value;
+            }
             fn is_normalized(&self) -> bool {
                 self.0 >= T::zero() && self.0 < Self::period()
             }
 
-            fn normalize(&self) -> $Struct<T> {
+            fn normalize(self) -> $Struct<T> {
                 if !self.is_normalized() {
                     let shifted = self.0 % Self::period();
                     if shifted < T::zero() {
@@ -204,7 +211,7 @@ macro_rules! impl_angle {
                         $Struct(shifted)
                     }
                 } else {
-                    self.clone()
+                    self
                 }
             }
 
@@ -510,6 +517,15 @@ mod test {
     use std::f64;
     use super::*;
 
+    #[test]
+    fn test_construct() {
+        assert_relative_eq!(Deg(50.0), Deg::new(50.0));
+        let mut a1 = Deg(100.0);
+        let scalar = a1.scalar();
+        a1.set_scalar(scalar + 150.0);
+        assert_relative_eq!(a1, Deg(250.0));
+    }
+
     #[test]
     fn test_convert() {
         assert_relative_eq!(ArcMinutes(120.0).into_angle(), Deg(2.0), epsilon=1e-6);
